/*
	*
	* Copyright 2019-2021 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This file was a part of PaperDE (https://gitlab.com/cubocore/paper/paperde)
	* Suitable modifications have been done to meet the needs of DesQ.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* at your option, any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QThread>

struct xdg_wm_base;
struct wl_seat;
struct wl_display;
struct wl_compositor;
struct zwlr_layer_shell_v1;
struct zwf_shell_manager_v2;
struct zwlr_foreign_toplevel_manager_v1;
struct zwlr_input_inhibit_manager_v1;
struct zwlr_input_inhibitor_v1;

class DesQWaylandRegistryPrivate;
class DesQLayerShell;
class DesQWindowManager;
class DesQInputInhibitManager;

class DesQWaylandRegistry : public QObject {
	Q_OBJECT

	public:
		enum ErrorType {
			EmptyXdgWmBase,
			EmptyWayfireShell,
			EmptyCompositor,
			EmptySeat,
			EmptyDesQLayerShell,
			EmptyToplevelManager,
			EmptyInputInhibitManager
		};
		Q_ENUM(ErrorType);

		DesQWaylandRegistry();
		DesQWaylandRegistry( wl_display *wlDisplay );
		~DesQWaylandRegistry();

		void initialize();

		wl_display *wlDisplay() const;
		wl_seat *wlSeat() const;
		wl_compositor *wlCompositor() const;

		xdg_wm_base *xdgWindowManagerBase() const;

		zwlr_layer_shell_v1 *wlrLayerShell() const;
		zwlr_foreign_toplevel_manager_v1 *wlrWindowManager() const;

		zwf_shell_manager_v2 *wayfireShell() const;

		DesQLayerShell *layerShell() const;
		DesQWindowManager *windowManager() const;

		DesQInputInhibitManager *inputInhibitManager() const;

	signals:
		void errorOccured( ErrorType et );

	protected:
		Q_DECLARE_PRIVATE( DesQWaylandRegistry );
		DesQWaylandRegistryPrivate *d_ptr;
};
