/*
	*
	* This file is a part of libpaper.
	* Library for handling various services related to Paper Shell.
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>

class DesQWindowHandle;
class DesQWindowHandlePrivate;
class DesQWindowManagerPrivate;

struct wl_seat;
struct wl_array;
struct wl_output;
struct zwlr_foreign_toplevel_handle_v1;
struct zwlr_foreign_toplevel_manager_v1;

typedef QList<DesQWindowHandle *> DesQWindowHandleList;

class DesQWindowManager : public QObject {
	Q_OBJECT;

	public:
		DesQWindowManager( zwlr_foreign_toplevel_manager_v1 *winMgr, wl_seat *seat, QObject *parent = nullptr );
		~DesQWindowManager();

		DesQWindowHandleList windowHandles() const;
		void activateHandle( DesQWindowHandle *handle );

	signals:
		void newTopLevelHandle( DesQWindowHandle *handle );
		void updateHandle( DesQWindowHandle *handle );

	protected:
		Q_DECLARE_PRIVATE( DesQWindowManager )
		DesQWindowManagerPrivate *d_ptr;

	private:
		void closeHandle( DesQWindowHandle *tlHandle );
};

class DesQWindowHandle : public QObject {
	Q_OBJECT;

	public:
		enum WindowState {
			None = ( 1 << 0 ),
			Activated = ( 1 << 1 ),
			Maximized = ( 1 << 2 ),
			Minimized = ( 1 << 3 )
		};
		Q_DECLARE_FLAGS( WindowStates, WindowState );

		DesQWindowHandle( zwlr_foreign_toplevel_handle_v1 *handle, QObject *parent = nullptr );
		~DesQWindowHandle();

		QString appID() const;
		QString appTitle() const;
		DesQWindowHandle::WindowStates windowStates() const;

		void activate( wl_seat *seat );
		void setMaximized();
		void unsetMaximized();
		void setMinimized();
		void unsetMinimized();
		void close();

	signals:
		/* App ID changed */
		void appIdUpdated( QString oldId, QString newId );

		/* Title and/or window states changed */
		void handleUpdated( DesQWindowHandle *handle );

		/* Window closed */
		void handleClosed( DesQWindowHandle *handle );

	protected:
		Q_DECLARE_PRIVATE( DesQWindowHandle )
		DesQWindowHandlePrivate *d_ptr;
};

Q_DECLARE_OPERATORS_FOR_FLAGS( DesQWindowHandle::WindowStates );
